#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <cv.h>
#include <highgui.h>

#include "myLightField.h"
#include "myLayerStack.h"
#include "myFocalStack.h"
#include "myLayerOptimize.h"
#include "myLayerOptimizeWithFocalStack.h"
#include "myPSNRcalculator.h"

int mode;
int varnum;
int hornum;
int layernum;
int *depth;
int frame;
double boost;
int iteration;
char loadfilename[128];
char savefilename[128];

void mode0(void);
void mode1(void);
void mode2(void);

int main(int argc, char *argv[]){
	
	if(argc != 2){
		printf("command line error.\n");
		exit(0);
	}
		
	FILE *fp;
	char s[256];

	fp = fopen(argv[1], "r");

	fscanf(fp, "%s\t%d", s, &mode);
	fscanf(fp, "%s\t%d", s, &varnum);
	fscanf(fp, "%s\t%d", s, &hornum);
	fscanf(fp, "%s\t%d", s, &layernum);

	depth = new int[layernum];
	for(int num = 0; num < layernum; num++){
		depth[num] = (-num + layernum / 2);
	}
	
	fscanf(fp, "%s\t%d", s, &frame);
	fscanf(fp, "%s\t%lf", s, &boost);
	fscanf(fp, "%s\t%d", s, &iteration);
	fscanf(fp, "%s\t%s", s, loadfilename);
	fscanf(fp, "%s\t%s", s, savefilename);

	fclose(fp);

	switch (mode) {
	case 0:
		mode0();
		break;
	case 1:
		mode1();
		break;
	case 2:
		mode2();
		break;
	default:
		printf("others\n");
		break;
	}

	return 0;
}

void mode0(void){
	printf("mode: 0\ninput: multi-view images\noptimize: multi-view images\noutput: layer patterns, displayed images\n");

	myLightField *tagLF = new myLightField();
	tagLF->loadMultiViewImages(hornum, varnum, loadfilename);
	int W = tagLF->get_W_size();
	int H = tagLF->get_H_size();
	int CH = tagLF->get_CH_size();
	int T = tagLF->get_T_size();
	int P = tagLF->get_P_size();

	myLayerStack *LS = new myLayerStack();
	LS->init(W, H, CH, frame, 1.0 / boost, layernum, depth);
	LS->setRandom();

	myLayerOptimize *Func = new myLayerOptimize();
	Func->optimizeLayerStackWithLightField(LS, tagLF, 0, iteration);

	myLightField *dispLF = tagLF->cloneLightField();
	dispLF->simulateLightFieldFromLayerStack(LS);

	LS->saveLayerStack(savefilename);
	dispLF->saveLightField_asImages(savefilename);

	printf("PSNR = %.2f\n", getPSNR_LightField(tagLF, dispLF, hornum));

	delete dispLF;
	delete LS;
	delete tagLF;
	delete Func;
}

void mode1(void){
	printf("mode: 1\ninput: multi-view images\noptimize: focal stack\noutput: focal stack, layer patterns, displayed images\n");

	myLightField *tagLF = new myLightField();
	tagLF->loadMultiViewImages(hornum, varnum, loadfilename);
	int W = tagLF->get_W_size();
	int H = tagLF->get_H_size();
	int CH = tagLF->get_CH_size();
	int T = tagLF->get_T_size();
	int P = tagLF->get_P_size();

	double *focusdepth = new double[layernum];
	for(int num = 0; num < layernum; num++){
		focusdepth[num] = (double)depth[num];
	}

	myFocalStack *FS = new myFocalStack();
	FS->init(W, H, T, P, CH, layernum, focusdepth);
	FS->setFocalStackFromLightField(tagLF);
	FS->saveFocalStack(savefilename);
	
	myLayerStack *LS = new myLayerStack();
	LS->init(W, H, CH, frame, 1.0 / boost, layernum, depth);
	LS->setRandom();

	myLayerOptimizeWithFocalStack *Func = new myLayerOptimizeWithFocalStack();
	Func->optimizeLayerStackWithFocalStack(LS, FS, iteration);

	myLightField *dispLF = tagLF->cloneLightField();
	dispLF->simulateLightFieldFromLayerStack(LS);

	LS->saveLayerStack(savefilename);
	dispLF->saveLightField_asImages(savefilename);

	printf("PSNR = %.2f\n", getPSNR_LightField(tagLF, dispLF, hornum));

	delete dispLF;
	delete FS;
	delete LS;
	delete tagLF;
	delete Func;
}

void mode2(void){
	printf("mode: 2\ninput: focal stack\noptimize: focal stack\noutput: layer patterns, displayed images\n");

	double *focusdepth = new double[layernum];
	for(int num = 0; num < layernum; num++){
		focusdepth[num] = (double)depth[num];
	}

	myFocalStack *FS = new myFocalStack();
	FS->loadFocalStack(hornum, varnum, layernum, focusdepth, loadfilename);
	
	int W = FS->get_W_size();
	int H = FS->get_H_size();
	int CH = FS->get_CH_size();
	int T = FS->get_T_size();
	int P = FS->get_P_size();

	myLayerStack *LS = new myLayerStack();
	LS->init(W, H, CH, frame, 1.0 / boost, layernum, depth);
	LS->setRandom();

	myLayerOptimizeWithFocalStack *Func = new myLayerOptimizeWithFocalStack();
	Func->optimizeLayerStackWithFocalStack(LS, FS, iteration);

	myLightField *dispLF = new myLightField();
	dispLF->init(W, H, T, P, CH);
	dispLF->simulateLightFieldFromLayerStack(LS);

	LS->saveLayerStack(savefilename);
	dispLF->saveLightField_asImages(savefilename);


	delete dispLF;
	delete FS;
	delete LS;
	delete Func;
}
