#include "myLayerOptimize.h"

myLayerOptimize::myLayerOptimize(void){
	this->mSetFlag = 0;
}

myLayerOptimize::~myLayerOptimize(void){
}

void myLayerOptimize::setParameter(int W, int H, int T, int P, int CH){
	this->mW = W;
	this->mH = H;
	this->mT = T;
	this->mP = P;
	this->mCH = CH;
	this->mSetFlag = 1;
	printf("set param to funcs.\n");
}

void myLayerOptimize::optimizeLayerStackWithLightField(myLayerStack *LS, myLightField *LF, int Approx, int ITER_MAX){

	if (this->mSetFlag == 0){
		setParameter(LF->get_W_size(), LF->get_H_size(), LF->get_T_size(), LF->get_P_size(), LF->get_CH_size());
	}

	int LayerNum = LS->get_LayerNum();

	for (int iter = 0; iter < ITER_MAX; iter++){

		for (int opt_lay = 0; opt_lay < LayerNum; opt_lay++){
			printf("iter = %d/%d, optimize lay_num = %d\n", iter + 1, ITER_MAX, opt_lay);
			optimizeOneLayerUseLightField(LS, LF, opt_lay, LayerNum, Approx);
			LS->showLayerStack(0.5);
			cvWaitKey(20);
		}
	}
}

void myLayerOptimize::optimizeOneLayerUseLightField(myLayerStack *LS, myLightField *LF, int opt_lay, int LayerNum, int Approx){

	myLightField *nowLF = new myLightField();
	nowLF = LF->cloneLightField();
	nowLF->simulateLightFieldFromLayerStack(LS);

	for (int y = 0; y < this->mH; y++){
		for (int x = 0; x < this->mW; x++){
			if (Approx == 1){
				UpdatePix_Approx(LS, LF, nowLF, opt_lay, LayerNum, x, y);
			}
			else{
				UpdatePix_Accurate(LS, LF, nowLF, opt_lay, LayerNum, x, y);
			}
		}
	}
	delete nowLF;
}

void myLayerOptimize::UpdatePix_Accurate(myLayerStack *LS, myLightField *LF, myLightField *nowLF, int opt_lay, int LayerNum, int x, int y){

	int Frame = LS->get_Frame_size();
	double Bright = LS->get_Bright();

	for (int frame = 0; frame < Frame; frame++){
		double numerator[3] = { 0.0, 0.0, 0.0 };
		double denominator[3] = { 0.000001, 0.000001, 0.000001 };

		for (int phi = 0; phi < this->mP; phi++){
			for (int theta = 0; theta < this->mT; theta++){

				int flag = 1;
				double A[3] = { 1.0, 1.0, 1.0 };

				for (int layer = 0; layer < LayerNum; layer++){
					if (layer != opt_lay){
						double lay_x, lay_y;
						get_XYcoord_onZ((double)LS->get_LayerZ(opt_lay), (double)x, (double)y, (double)theta, (double)phi, this->mT, this->mP, LS->get_LayerZ(layer), &lay_x, &lay_y);
						if (lay_x >= 0.0 && lay_x <= (double)(this->mW - 1) && lay_y >= 0.0 && lay_y <= (double)(this->mH - 1)){
							for (int ch = 0; ch < this->mCH; ch++){
								A[ch] *= LS->getValue((int)lay_x, (int)lay_y, ch, frame, layer);
							}
						}
						else{
							flag = 0;
							break;
						}
					}
				}

				double ray_t[3], ray_n[3];

				double x_z0, y_z0;
				get_XYcoord_onZ((double)LS->get_LayerZ(opt_lay), (double)x, (double)y, (double)theta, (double)phi, this->mT, this->mP, 0.0, &x_z0, &y_z0);
				if (x_z0 >= 0.0 && x_z0 <= (double)(this->mW - 1) && y_z0 >= 0.0 && y_z0 <= (double)(this->mH - 1)){
					for (int ch = 0; ch < this->mCH; ch++){
						ray_t[ch] = LF->getValue((int)x_z0, (int)y_z0, theta, phi, ch);
						ray_n[ch] = nowLF->getValue((int)x_z0, (int)y_z0, theta, phi, ch);
					}
				}
				else{
					flag = 0;
				}

				if (flag == 1){
					for (int ch = 0; ch < this->mCH; ch++){
						numerator[ch] += ray_t[ch] * Bright * A[ch];
						denominator[ch] += ray_n[ch] * Bright * A[ch];
					}
				}


			}
		}

		for (int ch = 0; ch < this->mCH; ch++){
			double value = LS->getValue(x, y, ch, frame, opt_lay);
			value *= (numerator[ch] / denominator[ch]);
			value = value * (value <= 1.0) + 1.0 * (value > 1.0);
			LS->setValue(x, y, ch, frame, opt_lay, value);
		}
	}
}

void myLayerOptimize::UpdatePix_Approx(myLayerStack *LS, myLightField *LF, myLightField *nowLF, int opt_lay, int LayerNum, int x, int y){

	int Frame = LS->get_Frame_size();
	double Bright = LS->get_Bright();

	for (int frame = 0; frame < Frame; frame++){
		double numerator[3] = { 0.0, 0.0, 0.0 };
		double denominator[3] = { 0.000001, 0.000001, 0.000001 };

		for (int phi = 0; phi < this->mP; phi++){
			for (int theta = 0; theta < this->mT; theta++){

				int flag = 1;
				double ray_t[3], ray_n[3];

				double x_z0, y_z0;
				get_XYcoord_onZ((double)LS->get_LayerZ(opt_lay), (double)x, (double)y, (double)theta, (double)phi, this->mT, this->mP, 0.0, &x_z0, &y_z0);
				if (x_z0 >= 0.0 && x_z0 <= (double)(this->mW - 1) && y_z0 >= 0.0 && y_z0 <= (double)(this->mH - 1)){
					for (int ch = 0; ch < this->mCH; ch++){
						ray_t[ch] = LF->getValue((int)x_z0, (int)y_z0, theta, phi, ch);
						ray_n[ch] = nowLF->getValue((int)x_z0, (int)y_z0, theta, phi, ch);
					}
				}
				else{
					flag = 0;
				}

				if (flag == 1){
					for (int ch = 0; ch < this->mCH; ch++){
						numerator[ch] += ray_t[ch] * Bright;
						denominator[ch] += ray_n[ch] * Bright;
					}
				}


			}
		}

		for (int ch = 0; ch < this->mCH; ch++){
			double value = LS->getValue(x, y, ch, frame, opt_lay);
			value *= (numerator[ch] / denominator[ch]);
			value = value * (value <= 1.0) + 1.0 * (value > 1.0);
			LS->setValue(x, y, ch, frame, opt_lay, value);
		}
	}
}
