#include <stdio.h>
#include <stdlib.h>
#include <GLUT/glut.h>
#include <highgui.h>
#include <cv.hpp>
using namespace cv;

#define WINDOW_NAME "Display Simulation"

#define WINSIZE 512
int LayerNum;
int Frame;
double Bright;
char savefilename[128];
const double LayerDis = 0.05;

void loadTxt(char *texfile);
void init(void);
void glut_display(void);
void glut_keyboard(unsigned char key, int x, int y);
void glut_mouse(int button, int state, int x, int y);
void glut_motion(int x, int y);
void glut_idle(void);
void draw_layer(void);
void draw_backlight(void);
void set_texture(void);

int TexSize;
double Angle1 = -0.005 * 20;
double Angle2 = -0.005 * 20;
int nowFrame = 0;
bool LeftButtonOn = false;
bool RightButtonOn = false;
int move = 1;

GLuint *Handle;

int main(int argc, char *argv[]){

	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE);
	glutInitWindowSize(WINSIZE, WINSIZE);
	glutCreateWindow(WINDOW_NAME);
	loadTxt(argv[1]);
	init();

	glutDisplayFunc(glut_display);
	glutKeyboardFunc(glut_keyboard);
	glutMouseFunc(glut_mouse);
	glutMotionFunc(glut_motion);
	glutIdleFunc(glut_idle);

	glutMainLoop();

	return 0;
}

void loadTxt(char *texfile){

	FILE *fp;
	char s[256];

	fp = fopen(texfile, "r");

	int mode, varnum, hornum, iteration;
	char loadfilename[128];
	
	fscanf(fp, "%s\t%d", s, &mode);
	fscanf(fp, "%s\t%d", s, &varnum);
	fscanf(fp, "%s\t%d", s, &hornum);
	fscanf(fp, "%s\t%d", s, &LayerNum);
	
	fscanf(fp, "%s\t%d", s, &Frame);
	fscanf(fp, "%s\t%lf", s, &Bright);
	fscanf(fp, "%s\t%d", s, &iteration);
	fscanf(fp, "%s\t%s", s, loadfilename);
	fscanf(fp, "%s\t%s", s, savefilename);

	fclose(fp);
}

void init(void){

	char Filename[128];
	sprintf(Filename, "%slayer-%d-%d.png", savefilename, 0, 0);
	Mat img_load = imread(Filename, 1);
	TexSize = img_load.cols;

	glClearColor(0.4, 0.4, 0.4, 0.0);

	Handle = new GLuint[LayerNum * Frame];
	glGenTextures(LayerNum * Frame, Handle);

	for (int i = 0; i < LayerNum * Frame; i++){

		glBindTexture(GL_TEXTURE_2D, Handle[i]);
		glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, TexSize, TexSize, 0, GL_RGB, GL_UNSIGNED_BYTE, NULL);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	}
	set_texture();
}

void set_texture(void){

	for (int layer = 0; layer < LayerNum; layer++){
		for (int frame = 0; frame < Frame; frame++){
			int i = layer * Frame + frame;

			char Filename[128];
			sprintf(Filename, "%slayer-%d-%d.png", savefilename, layer, frame);

			Mat img_load = imread(Filename, 1);
			cvtColor(img_load, img_load, CV_BGR2RGB);

			glBindTexture(GL_TEXTURE_2D, Handle[i]);
			glTexSubImage2D(GL_TEXTURE_2D, 0, 0, (TexSize - img_load.rows) / 2, img_load.cols, img_load.rows, GL_RGB, GL_UNSIGNED_BYTE, img_load.data);
		}
	}
}

void glut_keyboard(unsigned char key, int x, int y){

	switch (key){
	
	case 13:
		move = 1 - move;
		break;
	case 'q':
	case 'Q':
	case '\033':
		exit(0);
	}
	glutPostRedisplay();
}

void glut_mouse(int button, int state, int x, int y){

	if (button == GLUT_LEFT_BUTTON){
		if (state == GLUT_UP){
			LeftButtonOn = false;
		}
		else if (state == GLUT_DOWN){
			LeftButtonOn = true;
		}
	}

	if (button == GLUT_RIGHT_BUTTON){
		if (state == GLUT_UP){
			RightButtonOn = false;
		}
		else if (state == GLUT_DOWN){
			RightButtonOn = true;
		}
	}

}

void glut_motion(int x, int y){

	static int px = -1, py = -1;

	if (LeftButtonOn == true){

		if (px >= 0 && py >= 0){
			Angle1 += (double)-(x - px) / 40;
			Angle2 += (double)(y - py) / 40;
		}

		px = x;
		py = y;

	}
	else{
		px = -1;
		py = -1;
	}

	glutPostRedisplay();
}

void glut_display(void)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-1.0, 1.0, -1.0, 1.0, 0.001, 100.0);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(cos(Angle2) * sin(Angle1), sin(Angle2), cos(Angle2) * cos(Angle1), 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);

	double depth = -(double)LayerNum / 2.0 * LayerDis;

	glPushMatrix();
	glTranslatef(0.0, 0.0, depth);
	draw_backlight();
	glPopMatrix();
	depth += LayerDis;

	glEnable(GL_BLEND);
	glBlendFunc(GL_ZERO, GL_SRC_COLOR);

	for (int layer = LayerNum -1; layer >= 0; layer--){

		int i = layer * Frame + nowFrame;
		glBindTexture(GL_TEXTURE_2D, Handle[i]);

		glPushMatrix();
		glTranslatef(0.0, 0.0, depth);
		draw_layer();
		glPopMatrix();
		depth += LayerDis;
	}

	glDisable(GL_BLEND);

	glFlush();
	glDisable(GL_DEPTH_TEST);

	glutSwapBuffers();
}

void glut_idle(){

	nowFrame++;
	nowFrame %= Frame;

	static int cnt = 0;
	if(move == 1){
		if(cnt < 40){
			Angle1 += 0.005;
		}
		else if(cnt < 80){
			Angle2 += 0.005;
		}
		else if(cnt < 120){
			Angle1 -= 0.005;
		}
		else{
			Angle2 -= 0.005;
		}
		//printf("%d\n", cnt);
		cnt = (cnt + 1) % 160;
	}

	glutPostRedisplay();
}

void draw_layer(void){

	GLdouble pointA[] = { -0.8, -0.8, 0.0 };
	GLdouble pointB[] = { 0.8, -0.8, 0.0 };
	GLdouble pointC[] = { 0.8, 0.8, 0.0 };
	GLdouble pointD[] = { -0.8, 0.8, 0.0 };

	glColor4d(1.0, 1.0, 1.0, 1.0);

	glEnable(GL_TEXTURE_2D);

	glBegin(GL_POLYGON);
	glTexCoord2d(0.0, 1.0);
	glVertex3dv(pointA);
	glTexCoord2d(1.0, 1.0);
	glVertex3dv(pointB);
	glTexCoord2d(1.0, 0.0);
	glVertex3dv(pointC);
	glTexCoord2d(0.0, 0.0);
	glVertex3dv(pointD);
	glEnd();

	glDisable(GL_TEXTURE_2D);
}

void draw_backlight(void){

	GLdouble pointA[] = { -0.9, -0.9, 0.0 };
	GLdouble pointB[] = { 0.9, -0.9, 0.0 };
	GLdouble pointC[] = { 0.9, 0.9, 0.0 };
	GLdouble pointD[] = { -0.9, 0.9, 0.0 };

	glColor4d(1.0, 1.0, 1.0, 1.0);

	glBegin(GL_POLYGON);
	glVertex3dv(pointA);
	glVertex3dv(pointB);
	glVertex3dv(pointC);
	glVertex3dv(pointD);
	glEnd();
}
